@@ -14,14 +14,14 @@ import sys
 
 import psutil
 
-import saltfactories
+import saltfactories.utils.processes.helpers
+import saltfactories.utils.processes.salts as salt_factories
 from saltfactories.factories import master
 from saltfactories.factories import minion
 from saltfactories.factories import proxy
 from saltfactories.factories import syndic
 from saltfactories.utils import cli_scripts
 from saltfactories.utils import event_listener
-from saltfactories.utils import processes
 
 
 class SaltFactoriesManager(object):
@@ -38,7 +38,6 @@ class SaltFactoriesManager(object):
     def __init__(
         self,
         pytestconfig,
-        stats_processes,
         root_dir,
         log_server_port,
         log_server_level,
@@ -50,7 +49,42 @@ class SaltFactoriesManager(object):
         environ=None,
         slow_stop=True,
         start_timeout=10,
+        stats_processes=None,
     ):
+        """
+
+        Args:
+            pytestconfig (:fixture:`pytestconfig`):
+                PyTest `pytestconfig` fixture
+            root_dir:
+            log_server_port (int):
+            log_server_level (int):
+            executable (str) :
+                The path to the python executable to use to run python CLI scripts.
+                Defaults to :py:attr:`sys.executable`
+            code_dir (str):
+                The path to the code root directory of the project being tested. This is important for proper
+                code-coverage paths.
+            inject_coverage (bool):
+                Inject code-coverage related code in the generated CLI scripts
+            inject_sitecustomize (bool):
+                Inject code in the generated CLI scripts in order for our `sitecustumise.py` to be loaded by
+                subprocesses.
+            cwd (str):
+                The path to the current working directory
+            environ(dict):
+                A dictionary of `key`, `value` pairs to add to the environment.
+            slow_stop(bool):
+                Wether to terminate the processes by sending a :py:attr:`SIGTERM` signal or by calling
+                :py:meth:`~subprocess.Popen.terminate` on the sub-procecess.
+                When code coverage is enabled, one will want `slow_stop` set to `True` so that coverage data
+                can be written down to disk.
+            start_timeout(int):
+                The amount of time, in seconds, to wait, until a subprocess is considered as not started.
+            stats_processes (:py:class:`~collections.OrderedDict`):
+                This will be an `OrderedDict` instantiated on the :py:func:`~_pytest.hookspec.pytest_sessionstart`
+                hook accessible at `stats_processes` on the `session` attribute of the :fixture:`request`.
+        """
         self.pytestconfig = pytestconfig
         self.stats_processes = stats_processes
         self.root_dir = root_dir
@@ -74,6 +108,7 @@ class SaltFactoriesManager(object):
             "minions": {},
             "syndics": {},
             "proxy_minions": {},
+            "daemons": {},
         }
         self.event_listener = event_listener.EventListener()
         self.event_listener.start()
@@ -222,7 +257,7 @@ class SaltFactoriesManager(object):
             minion_config = self.configure_minion(request, minion_id, master_id=master_id)
 
         return self._start_daemon(
-            request, "salt-minion", minion_config, processes.SaltMinion, "minions", minion_id
+            request, "salt-minion", minion_config, salt_factories.SaltMinion, "minions", minion_id
         )
 
     def configure_master(self, request, master_id, order_masters=False, master_of_masters_id=None):
@@ -298,7 +333,7 @@ class SaltFactoriesManager(object):
             )
 
         return self._start_daemon(
-            request, "salt-master", master_config, processes.SaltMaster, "masters", master_id
+            request, "salt-master", master_config, salt_factories.SaltMaster, "masters", master_id
         )
 
     def configure_syndic(self, request, syndic_id, master_of_masters_id=None):
@@ -420,7 +455,7 @@ class SaltFactoriesManager(object):
             self.spawn_minion(request, syndic_id)
 
         return self._start_daemon(
-            request, "salt-syndic", syndic_config, processes.SaltSyndic, "syndics", syndic_id
+            request, "salt-syndic", syndic_config, salt_factories.SaltSyndic, "syndics", syndic_id
         )
 
     def configure_proxy_minion(self, request, proxy_minion_id, master_id=None):
@@ -497,12 +532,12 @@ class SaltFactoriesManager(object):
             request,
             "salt-proxy",
             proxy_minion_config,
-            processes.SaltProxyMinion,
+            salt_factories.SaltProxyMinion,
             "proxy_minions",
             proxy_minion_id,
         )
 
-    def get_salt_cli(self, request, master_id):
+    def get_salt_cli(self, request, master_id, **cli_kwargs):
         """
         Return a `salt` CLI process
         """
@@ -514,9 +549,11 @@ class SaltFactoriesManager(object):
             inject_coverage=self.inject_coverage,
             inject_sitecustomize=self.inject_sitecustomize,
         )
-        return processes.SaltCLI(script_path, config=self.cache["masters"][master_id].config)
+        return salt_factories.SaltCLI(
+            script_path, config=self.cache["masters"][master_id].config, **cli_kwargs
+        )
 
-    def get_salt_call_cli(self, request, minion_id):
+    def get_salt_call_cli(self, request, minion_id, **cli_kwargs):
         """
         Return a `salt-call` CLI process
         """
@@ -529,22 +566,23 @@ class SaltFactoriesManager(object):
             inject_sitecustomize=self.inject_sitecustomize,
         )
         try:
-            return processes.SaltCallCLI(
-                script_path, config=self.cache["minions"][minion_id].config
+            return salt_factories.SaltCallCLI(
+                script_path, config=self.cache["minions"][minion_id].config, **cli_kwargs
             )
         except KeyError:
             try:
-                return processes.SaltCallCLI(
+                return salt_factories.SaltCallCLI(
                     script_path,
                     base_script_args=["--proxyid={}".format(minion_id)],
                     config=self.cache["proxy_minions"][minion_id].config,
+                    **cli_kwargs
                 )
             except KeyError:
                 raise KeyError(
                     "Could not find {} in the minions or proxy minions caches".format(minion_id)
                 )
 
-    def get_salt_run(self, request, master_id):
+    def get_salt_run(self, request, master_id, **cli_kwargs):
         """
         Return a `salt-run` CLI process
         """
@@ -556,9 +594,11 @@ class SaltFactoriesManager(object):
             inject_coverage=self.inject_coverage,
             inject_sitecustomize=self.inject_sitecustomize,
         )
-        return processes.SaltRunCLI(script_path, config=self.cache["masters"][master_id].config)
+        return salt_factories.SaltRunCLI(
+            script_path, config=self.cache["masters"][master_id].config, **cli_kwargs
+        )
 
-    def get_salt_cp(self, request, master_id):
+    def get_salt_cp(self, request, master_id, **cli_kwargs):
         """
         Return a `salt-cp` CLI process
         """
@@ -570,9 +610,11 @@ class SaltFactoriesManager(object):
             inject_coverage=self.inject_coverage,
             inject_sitecustomize=self.inject_sitecustomize,
         )
-        return processes.SaltCpCLI(script_path, config=self.cache["masters"][master_id].config)
+        return salt_factories.SaltCpCLI(
+            script_path, config=self.cache["masters"][master_id].config, **cli_kwargs
+        )
 
-    def get_salt_key(self, request, master_id):
+    def get_salt_key(self, request, master_id, **cli_kwargs):
         """
         Return a `salt-key` CLI process
         """
@@ -584,7 +626,46 @@ class SaltFactoriesManager(object):
             inject_coverage=self.inject_coverage,
             inject_sitecustomize=self.inject_sitecustomize,
         )
-        return processes.SaltKeyCLI(script_path, config=self.cache["masters"][master_id].config)
+        return salt_factories.SaltKeyCLI(
+            script_path, config=self.cache["masters"][master_id].config, **cli_kwargs
+        )
+
+    def spawn_daemon(
+        self,
+        request,
+        script_name,
+        daemon_class,
+        daemon_id,
+        environ=None,
+        cwd=None,
+        slow_stop=None,
+        **extra_daemon_class_kwargs
+    ):
+        """
+        Start a non-salt daemon
+        """
+        if environ is None:
+            environ = self.environ
+        if cwd is None:
+            cwd = self.cwd
+        if slow_stop is None:
+            slow_stop = False
+        proc = saltfactories.utils.processes.helpers.start_daemon(
+            script_name,
+            daemon_class,
+            start_timeout=self.start_timeout,
+            slow_stop=slow_stop,
+            environ=environ,
+            cwd=cwd,
+            max_attempts=3,
+            **extra_daemon_class_kwargs
+        )
+        self.cache["daemons"][daemon_id] = proc
+        if self.stats_processes:
+            self.stats_processes[proc.get_display_name()] = psutil.Process(proc.pid)
+        request.addfinalizer(proc.terminate)
+        request.addfinalizer(lambda: self.cache["daemons"].pop(daemon_id))
+        return proc
 
     def _start_daemon(
         self, request, script_name, daemon_config, daemon_class, cache_key, daemon_id
@@ -600,10 +681,10 @@ class SaltFactoriesManager(object):
             inject_coverage=self.inject_coverage,
             inject_sitecustomize=self.inject_sitecustomize,
         )
-        proc = processes.start_daemon(
-            daemon_config,
+        proc = saltfactories.utils.processes.helpers.start_daemon(
             script_path,
             daemon_class,
+            config=daemon_config,
             start_timeout=self.start_timeout,
             slow_stop=self.slow_stop,
             environ=self.environ,
@@ -613,8 +694,7 @@ class SaltFactoriesManager(object):
         )
         self.cache[cache_key][daemon_id] = proc
         if self.stats_processes:
-            process_name = proc.log_prefix.strip().lstrip("[").rstrip("]")
-            self.stats_processes[process_name] = psutil.Process(proc.pid)
+            self.stats_processes[proc.get_display_name()] = psutil.Process(proc.pid)
         request.addfinalizer(proc.terminate)
         request.addfinalizer(lambda: self.cache[cache_key].pop(daemon_id))
         return proc
